@@ -24,11 +24,14 @@ of frames for different uses.
 from nuitka.PythonVersions import python_version
 
 from . import Emission
+from .CodeHelpers import _generateStatementSequenceCode
+from .ErrorCodes import getFrameVariableTypeDescriptionCode
 from .ExceptionCodes import getTracebackMakingIdentifier
-from .Helpers import _generateStatementSequenceCode
 from .Indentation import indented
+from .LabelCodes import getGotoCode, getLabelCode
 from .ModuleCodes import getModuleAccessCode
 from .templates.CodeTemplatesFrames import (
+    template_frame_attach_locals,
     template_frame_guard_cache_decl,
     template_frame_guard_frame_decl,
     template_frame_guard_full_block,
@@ -59,6 +62,8 @@ def getFrameLocalsStorageSize(type_descriptions):
     result = candidates.pop()
 
     while candidates:
+        # assert False, (type_descriptions, context.frame_variables_stack[-1])
+
         result = "MAX( %s, %s )" % (result, candidates.pop())
 
     return result
@@ -67,39 +72,41 @@ def getFrameLocalsStorageSize(type_descriptions):
 def generateStatementsFrameCode(statement_sequence, emit, context):
     # This is a wrapper that provides also handling of frames, which got a
     # lot of variants and details, therefore lots of branches.
-    # pylint: disable=too-many-branches,too-many-statements
+    # pylint: disable=too-many-branches
 
     context.pushCleanupScope()
 
-    provider = statement_sequence.getParentVariableProvider()
     guard_mode = statement_sequence.getGuardMode()
 
+    code_object = statement_sequence.getCodeObject()
+    code_identifier = context.getCodeObjectHandle(
+        code_object = code_object
+    )
+
     parent_exception_exit = context.getExceptionEscape()
 
-    # Allow stacking of frame handles.
-    old_frame_handle = context.getFrameHandle()
+    # For nested frames, make sure to restore set the type description.
+    if context.getFrameHandle() is not None:
+        real_parent_exception_exit = parent_exception_exit
+        parent_exception_exit = context.allocateLabel("nested_frame_exit")
 
-    if guard_mode != "pass_through":
-        if provider.isExpressionGeneratorObjectBody():
-            context.setFrameHandle("generator->m_frame")
-        elif provider.isExpressionCoroutineObjectBody():
-            context.setFrameHandle("coroutine->m_frame")
-        elif provider.isExpressionAsyncgenObjectBody():
-            context.setFrameHandle("asyncgen->m_frame")
-        elif provider.isCompiledPythonModule():
-            context.setFrameHandle("frame_module")
-        else:
-            context.setFrameHandle("frame_function")
-
-        context.setExceptionEscape(
-            context.allocateLabel("frame_exception_exit")
-        )
+    if statement_sequence.hasStructureMember():
+        frame_identifier = "%s->m_frame" % context.getContextObjectName()
     else:
-        context.setFrameHandle("((struct Nuitka_FrameObject *)PyThreadState_GET()->frame)")
+        frame_identifier = code_identifier.replace("codeobj_", "frame_")
+
+    # Allow stacking of frame handles.
+    frame_identifier = context.pushFrameHandle(
+        frame_identifier
+    )
+
+    context.setExceptionEscape(
+        context.allocateLabel("frame_exception_exit")
+    )
 
     needs_preserve = statement_sequence.needsFrameExceptionPreserving()
 
-    if statement_sequence.mayReturn() and guard_mode != "pass_through":
+    if statement_sequence.mayReturn():
         parent_return_exit = context.getReturnTarget()
 
         context.setReturnTarget(
@@ -108,15 +115,25 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     else:
         parent_return_exit = None
 
-    if guard_mode != "pass_through":
-        pushed_frame_variables = statement_sequence.mayRaiseException(BaseException)
+    # We need to define that all the variables needs to be pushed. We do not
+    # have a flag that says "always NULL" for variables. With efficient NULL
+    # passing however (not at all, TODO), that doesn't matter much.
+    local_variables = statement_sequence.getParentVariableProvider().getLocalVariables()
 
-        if pushed_frame_variables:
-            context.pushFrameVariables(
-                statement_sequence.getCodeObject().getVarNames()
-            )
-    else:
-        pushed_frame_variables = None
+    def search(variable_name):
+        for local_variable in local_variables:
+            if local_variable.getName() == variable_name:
+                return local_variable
+
+        return None
+
+    context.pushFrameVariables(
+        tuple(
+            search(variable_name)
+            for variable_name in
+            code_object.getVarNames()
+        )
+    )
 
     # Now generate the statements code into a local buffer, to we can wrap
     # the frame stuff around it.
@@ -141,18 +158,11 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     type_descriptions = context.getFrameVariableTypeDescriptions()
 
     if guard_mode == "generator":
-        # Only these two use this.
-        assert provider.isExpressionGeneratorObjectBody() or \
-               provider.isExpressionCoroutineObjectBody() or \
-               provider.isExpressionAsyncgenObjectBody()
-
         # TODO: This case should care about "needs_preserve", as for
         # Python3 it is actually not a stub of empty code.
 
         getFrameGuardLightCode(
-            code_identifier       = statement_sequence.getCodeObjectHandle(
-                context = context
-            ),
+            code_identifier       = code_identifier,
             type_descriptions     = type_descriptions,
             codes                 = local_emit.codes,
             parent_exception_exit = parent_exception_exit,
@@ -162,19 +172,10 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
             emit                  = emit,
             context               = context
         )
-    elif guard_mode == "pass_through":
-        # This case does not care about "needs_preserve", as for that kind
-        # of frame, it is an empty code stub anyway.
-        local_emit.emitTo(emit)
     elif guard_mode == "full":
-        assert provider.isExpressionFunctionBody() or \
-               provider.isExpressionClassBody()
-
         getFrameGuardHeavyCode(
             frame_identifier      = context.getFrameHandle(),
-            code_identifier       = statement_sequence.getCodeObjectHandle(
-                context
-            ),
+            code_identifier       = code_identifier,
             type_descriptions     = type_descriptions,
             parent_exception_exit = parent_exception_exit,
             parent_return_exit    = parent_return_exit,
@@ -188,9 +189,7 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     elif guard_mode == "once":
         getFrameGuardOnceCode(
             frame_identifier      = context.getFrameHandle(),
-            code_identifier       = statement_sequence.getCodeObjectHandle(
-                context = context
-            ),
+            code_identifier       = code_identifier,
             parent_exception_exit = parent_exception_exit,
             parent_return_exit    = parent_return_exit,
             frame_exception_exit  = frame_exception_exit,
@@ -203,33 +202,54 @@ def generateStatementsFrameCode(statement_sequence, emit, context):
     else:
         assert False, guard_mode
 
-    if pushed_frame_variables:
-        context.popFrameVariables()
+    context.popFrameVariables()
+    context.popFrameHandle()
+
+    # For nested frames, make sure to restore set the type description.
+    if context.getFrameHandle() is not None:
+        label = context.allocateLabel("skip_nested_handling")
+        getGotoCode(label, emit)
+        getLabelCode(parent_exception_exit, emit)
+        emit(getFrameVariableTypeDescriptionCode(context))
+        getGotoCode(real_parent_exception_exit, emit)
+        getLabelCode(label, emit)
+
+        parent_exception_exit = real_parent_exception_exit
+
+    context.popCleanupScope()
 
     context.setExceptionEscape(parent_exception_exit)
 
     if frame_return_exit is not None:
         context.setReturnTarget(parent_return_exit)
 
-    context.setFrameHandle(old_frame_handle)
-
-    context.popCleanupScope()
 
 
 def getTypeSizeOf(type_indicator):
-    if type_indicator in ('O', 'o'):
-        return "sizeof(PyObject *)"
-    elif type_indicator == 'c':
-        return "sizeof(struct Nuitka_CellObject *)"
-    elif type_indicator == 'N':
+    if type_indicator in ('O', 'o', 'N', 'c'):
         return "sizeof(void *)"
+    elif type_indicator == 'b':
+        return "sizeof(nuitka_bool)"
     else:
         assert False, type_indicator
 
 
+def getFrameAttachLocalsCode(context, frame_identifier):
+    frame_variable_codes = context.getFrameVariableCodeNames()
+    frame_variable_codes = ",\n    ".join(frame_variable_codes)
+    if frame_variable_codes:
+        frame_variable_codes = ",\n    " + frame_variable_codes
+
+
+    return template_frame_attach_locals % {
+        "frame_identifier"      : frame_identifier,
+        "type_description"      : context.getFrameVariableTypeDescriptionName(),
+        "frame_variable_refs"   : frame_variable_codes
+    }
+
+
 def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
-                           type_descriptions,
-                           needs_preserve, parent_exception_exit,
+                           type_descriptions, needs_preserve, parent_exception_exit,
                            parent_return_exit, frame_exception_exit,
                            frame_return_exit, emit, context):
     # We really need this many parameters here.
@@ -247,15 +267,13 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
         }
     )
 
-
-
     emit(
         template_frame_guard_full_block % {
             "frame_identifier"  : frame_identifier,
             "code_identifier"   : code_identifier,
             "locals_size"       : getFrameLocalsStorageSize(type_descriptions),
             "codes"             : indented(codes, 0),
-            "module_identifier" : getModuleAccessCode(context = context),
+            "module_identifier" : getModuleAccessCode(context),
             "no_exception_exit" : no_exception_exit,
             "needs_preserve"    : 1 if needs_preserve else 0,
         }
@@ -272,12 +290,6 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
         )
 
     if frame_exception_exit is not None:
-        frame_variable_codes = context.getFrameVariableCodeNames()
-
-        if frame_variable_codes:
-            frame_variable_codes = ',' + frame_variable_codes
-        else:
-            frame_variable_codes = ""
 
         emit(
             template_frame_guard_full_exception_handler % {
@@ -288,8 +300,7 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
                                           ),
                 "parent_exception_exit" : parent_exception_exit,
                 "frame_exception_exit"  : frame_exception_exit,
-                "type_description"      : "type_description" if context.needsFrameVariableTypeDescription() else '""',
-                "frame_variable_refs"   : frame_variable_codes,
+                "attach_locals"         : getFrameAttachLocalsCode(context, frame_identifier),
                 "needs_preserve"        : 1 if needs_preserve else 0,
             }
         )
@@ -317,7 +328,7 @@ def getFrameGuardOnceCode(frame_identifier, code_identifier,
             "frame_identifier"      : frame_identifier,
             "code_identifier"       : code_identifier,
             "codes"                 : indented(codes, 0),
-            "module_identifier"     : getModuleAccessCode(context = context),
+            "module_identifier"     : getModuleAccessCode(context),
             "tb_making"             : getTracebackMakingIdentifier(
                                          context     = context,
                                          lineno_name = "exception_lineno"
@@ -339,24 +350,25 @@ def getFrameGuardLightCode(code_identifier, codes, parent_exception_exit,
     context.markAsNeedsExceptionVariables()
 
     context_identifier = context.getContextObjectName()
-    frame_identifier = "frame_" + context_identifier
 
     context.addFrameDeclaration(
         template_frame_guard_cache_decl % {
-            "frame_identifier" : frame_identifier,
+            "frame_identifier" : "frame_" + context_identifier,
         }
     )
 
     no_exception_exit = context.allocateLabel("frame_no_exception")
 
+    frame_identifier = context_identifier + "->m_frame"
+
     emit(
         template_frame_guard_generator % {
             "context_identifier"     : context_identifier,
-            "frame_cache_identifier" : "cache_" + frame_identifier,
+            "frame_cache_identifier" : "cache_frame_" + context_identifier,
             "code_identifier"        : code_identifier,
             "locals_size"            : getFrameLocalsStorageSize(type_descriptions),
             "codes"                  : indented(codes, 0),
-            "module_identifier"      : getModuleAccessCode(context = context),
+            "module_identifier"      : getModuleAccessCode(context),
             "no_exception_exit"      : no_exception_exit,
         }
     )
@@ -371,23 +383,17 @@ def getFrameGuardLightCode(code_identifier, codes, parent_exception_exit,
         )
 
     if frame_exception_exit is not None:
-        frame_variable_codes = context.getFrameVariableCodeNames()
-
-        if frame_variable_codes:
-            frame_variable_codes = ',' + frame_variable_codes
-        else:
-            frame_variable_codes = ""
-
         emit(
             template_frame_guard_generator_exception_handler % {
                 "frame_identifier"       : "%s->m_frame" % context_identifier,
-                "frame_cache_identifier" : "cache_" + frame_identifier,
+                "frame_cache_identifier" : "cache_frame_" + context_identifier,
                 "tb_making"              : getTracebackMakingIdentifier(
                                                context     = context,
                                                lineno_name = "exception_lineno"
                                            ),
-                "type_description"       : "type_description" if context.needsFrameVariableTypeDescription() else '""',
-                "frame_variable_refs"    : frame_variable_codes,
+                "attach_locals"          : indented(
+                    getFrameAttachLocalsCode(context, frame_identifier)
+                ),
                 "frame_exception_exit"   : frame_exception_exit,
                 "parent_exception_exit"  : parent_exception_exit
             }
