@@ -19,15 +19,9 @@
 
 """
 
-from nuitka.PythonVersions import python_version
-
+from .CodeHelpers import generateExpressionCode
 from .Emission import SourceCodeCollector
-from .ErrorCodes import (
-    getCheckObjectCode,
-    getErrorFormatExitBoolCode,
-    getErrorFormatExitCode
-)
-from .Helpers import generateExpressionCode
+from .ErrorCodes import getCheckObjectCode, getNameReferenceErrorCode
 from .Indentation import indented
 from .templates.CodeTemplatesVariables import (
     template_del_global_unclear,
@@ -37,13 +31,10 @@ from .templates.CodeTemplatesVariables import (
 
 
 def generateAssignmentVariableCode(statement, emit, context):
-    variable_ref  = statement.getTargetVariableRef()
-    value         = statement.getAssignSource()
-
     tmp_name = context.allocateTempName("assign_source")
 
     generateExpressionCode(
-        expression = value,
+        expression = statement.getAssignSource(),
         to_name    = tmp_name,
         emit       = emit,
         context    = context
@@ -51,8 +42,8 @@ def generateAssignmentVariableCode(statement, emit, context):
 
     getVariableAssignmentCode(
         tmp_name      = tmp_name,
-        variable      = variable_ref.getVariable(),
-        version       = variable_ref.getVariableVersion(),
+        variable      = statement.getVariable(),
+        version       = statement.getVariableVersion(),
         needs_release = statement.needsReleasePreviousValue(),
         in_place      = statement.inplace_suspect,
         emit          = emit,
@@ -68,9 +59,8 @@ def generateDelVariableCode(statement, emit, context):
         statement.getSourceReference()
     )
 
-
     getVariableDelCode(
-        variable    = statement.getTargetVariableRef().getVariable(),
+        variable    = statement.getVariable(),
         new_version = statement.variable_trace.getVersion(),
         old_version = statement.previous_trace.getVersion(),
         tolerant    = statement.isTolerant(),
@@ -131,7 +121,17 @@ def getLocalVariableCodeType(context, variable, version):
     user = context.getOwner()
     owner = variable.getOwner()
 
-    variable_trace = user.trace_collection.getVariableTrace(variable, version)
+    user = user.getEntryPoint()
+
+    prefix = ""
+
+    if owner.isExpressionOutlineFunction() or owner.isExpressionClassBody():
+        entry_point = owner.getEntryPoint()
+
+        prefix = "outline_%d_" % entry_point.getTraceCollection().getOutlineFunctions().index(owner)
+        owner = entry_point
+
+    variable_trace = user.getTraceCollection().getVariableTrace(variable, version)
 
     c_type = variable_trace.getPickedCType(context)
 
@@ -140,18 +140,20 @@ def getLocalVariableCodeType(context, variable, version):
             in_context = False,
             variable   = variable
         )
+
+        result = prefix + result
     elif context.isForDirectCall():
 
         if user.isExpressionGeneratorObjectBody():
-            closure_index = user.getClosureVariables().index(variable)
+            closure_index = user.getClosureVariableIndex(variable)
 
             result = "generator->m_closure[%d]" % closure_index
         elif user.isExpressionCoroutineObjectBody():
-            closure_index = user.getClosureVariables().index(variable)
+            closure_index = user.getClosureVariableIndex(variable)
 
             result = "coroutine->m_closure[%d]" % closure_index
         elif user.isExpressionAsyncgenObjectBody():
-            closure_index = user.getClosureVariables().index(variable)
+            closure_index = user.getClosureVariableIndex(variable)
 
             result = "asyncgen->m_closure[%d]" % closure_index
         else:
@@ -159,8 +161,10 @@ def getLocalVariableCodeType(context, variable, version):
                 in_context = True,
                 variable   = variable
             )
+
+            result = prefix + result
     else:
-        closure_index = user.getClosureVariables().index(variable)
+        closure_index = user.getClosureVariableIndex(variable)
 
         if user.isExpressionGeneratorObjectBody():
             result = "generator->m_closure[%d]" % closure_index
@@ -169,6 +173,9 @@ def getLocalVariableCodeType(context, variable, version):
         elif user.isExpressionAsyncgenObjectBody():
             result = "asyncgen->m_closure[%d]" % closure_index
         else:
+            # TODO: If this were context.getContextObjectName() this would be
+            # a one liner.
+
             result = "self->m_closure[%d]" % closure_index
 
     return result, c_type
@@ -255,23 +262,11 @@ def _generateModuleVariableAccessCode(to_name, variable_name, needs_check,
         }
     )
     if needs_check:
-
-        if python_version < 340 and \
-           not context.isCompiledPythonModule() and \
-           not context.getOwner().isExpressionClassBody():
-            error_message = "global name '%s' is not defined"
-        else:
-            error_message = "name '%s' is not defined"
-
-        getErrorFormatExitCode(
-            check_name = to_name,
-            exception  = "PyExc_NameError",
-            args       = (
-                error_message,
-                variable_name
-            ),
-            emit       = emit,
-            context    = context
+        getNameReferenceErrorCode(
+            variable_name = variable_name,
+            condition     = "%s == NULL" % to_name,
+            emit          = emit,
+            context       = context
         )
     else:
         getCheckObjectCode(to_name, emit)
@@ -294,7 +289,7 @@ def generateLocalsDictVariableRefCode(to_name, expression, emit, context):
 
     emit(
         template_read_maybe_local_unclear % {
-            "locals_dict" : "locals_dict",
+            "locals_dict" : context.getLocalsDictName(),
             "fallback"    : indented(fallback_emit.codes),
             "tmp_name"    : to_name,
             "var_name"    : context.getConstantCode(
@@ -345,17 +340,11 @@ def getVariableDelCode(variable, old_version, new_version, tolerant,
 
         # TODO: Apply needs_check for module variables too.
         if check:
-            getErrorFormatExitBoolCode(
-                condition = "%s == -1" % res_name,
-                exception = "PyExc_NameError",
-                args      = (
-                    "%sname '%s' is not defined" % (
-                        "global " if not context.isCompiledPythonModule() else "",
-                        variable.getName()
-                    ),
-                ),
-                emit      = emit,
-                context   = context
+            getNameReferenceErrorCode(
+                variable_name = variable.getName(),
+                condition     = "%s == -1" % res_name,
+                emit          = emit,
+                context       = context
             )
     elif variable.isLocalVariable():
         variable_code_name, variable_c_type = getLocalVariableCodeType(context, variable, old_version)
