@@ -24,17 +24,9 @@ check it many times.
 """
 from nuitka.tree import Operations
 
-from .FinalizeClosureTaking import FinalizeClassClosure
 from .FinalizeMarkups import FinalizeMarkups
 
 
 def prepareCodeGeneration(tree):
     visitor = FinalizeMarkups()
     Operations.visitTree(tree, visitor)
-    for function in tree.getUsedFunctions():
-        Operations.visitTree(function, visitor)
-
-    visitor = FinalizeClassClosure()
-    for function in tree.getUsedFunctions():
-        if function.hasFlag("has_super"):
-            Operations.visitFunction(function, visitor)
