@@ -43,7 +43,7 @@ from .ExpressionBases import CompileTimeConstantExpressionBase
 class ExpressionBuiltinRefBase(CompileTimeConstantExpressionBase):
     # Base classes can be abstract, pylint: disable=abstract-method
 
-    __slots__ = "builtin_name",
+    __slots__ = ("builtin_name",)
 
     def __init__(self, builtin_name, source_ref):
         CompileTimeConstantExpressionBase.__init__(
@@ -143,7 +143,7 @@ class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
 
         if self.builtin_name in ("dir", "eval", "exec", "execfile", "locals", "vars"):
             # Just inform the collection that all has escaped.
-            trace_collection.onLocalsUsage()
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return new_node, tags, message
 
