@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,10 +19,12 @@
 
 """
 
-from .NodeBases import (
-    ExpressionChildrenHavingBase,
-    SideEffectsFromChildrenMixin
-)
+import functools
+
+from nuitka.PythonVersions import needsSetLiteralReverseInsertion
+
+from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeBases import SideEffectsFromChildrenMixin
 from .NodeMakingHelpers import (
     getComputationResult,
     makeStatementOnlyNodesFromExpressions,
@@ -67,7 +69,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
     getElements = ExpressionChildrenHavingBase.childGetter("elements")
 
     def getSimulator(self):
-        # Abstract method, pylint: disable=R0201
+        # Abstract method, pylint: disable=no-self-use
         return None
 
     def computeExpression(self, trace_collection):
@@ -91,7 +93,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
         simulator = self.getSimulator()
         assert simulator is not None
 
-        # The simulator is in fact callable if not None, pylint: disable=E1102
+        # The simulator is in fact callable if not None, pylint: disable=not-callable
         return getComputationResult(
             node        = self,
             computation = lambda : simulator(
@@ -256,3 +258,17 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
 
         return iter_node, "new_expression", """\
 Iteration over set reduced to tuple."""
+
+
+class ExpressionMakeSetLiteral(ExpressionMakeSet):
+    kind = "EXPRESSION_MAKE_SET_LITERAL"
+
+    def getSimulator(self):
+        if needsSetLiteralReverseInsertion():
+            @functools.wraps(set)
+            def mySet(value):
+                return set(reversed(tuple(value)))
+
+            return mySet
+        else:
+            return set
