@@ -26,9 +26,10 @@ them.
 
 """
 
-from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.PythonVersions import python_version
 
+from .CodeObjectSpecs import CodeObjectSpec
+from .FutureSpecs import fromFlags
 from .StatementNodes import StatementsSequence
 
 
@@ -50,8 +51,7 @@ def checkFrameStatements(value):
 
 
 
-class StatementsFrame(StatementsSequence):
-    kind = "STATEMENTS_FRAME"
+class StatementsFrameBase(StatementsSequence):
 
     checkers = {
         "statements" : checkFrameStatements
@@ -71,9 +71,11 @@ class StatementsFrame(StatementsSequence):
 
         self.needs_frame_exception_preserve = False
 
+    def isStatementsFrame(self):
+        return True
+
     def getDetails(self):
         result = {
-            "guard_mode"  : self.guard_mode,
             "code_object" : self.code_object
         }
 
@@ -82,11 +84,8 @@ class StatementsFrame(StatementsSequence):
         return result
 
     def getDetailsForDisplay(self):
-        result = {
-            "guard_mode"  : self.guard_mode,
-        }
-
-        result.update(StatementsSequence.getDetails(self))
+        result = StatementsSequence.getDetails(self)
+        result.update()
 
         result.update(self.code_object.getDetails())
 
@@ -100,6 +99,8 @@ class StatementsFrame(StatementsSequence):
         for key, value in args.items():
             if key.startswith("co_"):
                 code_object_args[key] = value
+            elif key == "code_flags":
+                code_object_args["future_spec"] = fromFlags(args["code_flags"])
             else:
                 other_args[key] = value
 
@@ -116,23 +117,20 @@ class StatementsFrame(StatementsSequence):
 
     def needsExceptionFramePreservation(self):
         if python_version < 300:
-            preserving = ("full", "once")
+            return self.guard_mode != "generator"
         else:
-            preserving = ("full", "once", "generator")
-
-        return self.guard_mode in preserving
+            return True
 
     def getVarNames(self):
         return self.code_object.getVarNames()
 
     def updateLocalNames(self):
-        """ For use during variable closure phase.
+        """ For use during variable closure phase. Finalize attributes.
 
         """
         provider = self.getParentVariableProvider()
 
-        if not provider.isCompiledPythonModule() and \
-           self.code_object is not None:
+        if not provider.isCompiledPythonModule():
             self.code_object.updateLocalNames(
                 [
                     variable.getName() for
@@ -141,6 +139,34 @@ class StatementsFrame(StatementsSequence):
                 ]
             )
 
+        entry_point = provider.getEntryPoint()
+
+        is_optimized = not entry_point.isCompiledPythonModule() and \
+                       not entry_point.isExpressionClassBody() and \
+                       not entry_point.hasLocalsDict()
+
+        self.code_object.setFlagIsOptimizedValue(is_optimized)
+
+        new_locals = not provider.isCompiledPythonModule() and \
+                     (python_version < 340 or (
+                     not provider.isExpressionClassBody() and \
+                     not provider.hasLocalsDict()))
+
+        self.code_object.setFlagNewLocalsValue(new_locals)
+
+        if provider.isExpressionGeneratorObjectBody() or \
+           provider.isExpressionCoroutineObjectBody() or \
+           provider.isExpressionAsyncgenObjectBody():
+            closure_provider = provider.getParentVariableProvider()
+        else:
+            closure_provider = provider
+
+        has_closure  = closure_provider.isExpressionFunctionBody() and \
+                       closure_provider.getClosureVariables() != () and \
+                       not closure_provider.isExpressionClassBody()
+
+        self.code_object.setFlagHasClosureValue(has_closure)
+
     def markAsFrameExceptionPreserving(self):
         self.needs_frame_exception_preserve = True
 
@@ -150,45 +176,6 @@ class StatementsFrame(StatementsSequence):
     def getCodeObject(self):
         return self.code_object
 
-    def getCodeObjectHandle(self, context):
-        provider = self.getParentVariableProvider()
-
-        is_optimized = not provider.isCompiledPythonModule() and \
-                       not provider.isExpressionClassBody() and \
-                       not provider.hasLocalsDict()
-
-        new_locals = not provider.isCompiledPythonModule() and \
-                     (python_version < 340 or (
-                     not provider.isExpressionClassBody() and \
-                     not provider.hasLocalsDict()))
-
-        if provider.isCompiledPythonModule():
-            line_number = 1
-        else:
-            line_number = self.source_ref.getLineNumber()
-
-        if provider.isExpressionGeneratorObjectBody() or \
-           provider.isExpressionCoroutineObjectBody() or \
-           provider.isExpressionAsyncgenObjectBody():
-            closure_provider = provider.getParentVariableProvider()
-        else:
-            closure_provider = provider
-
-        parent_module = self.getParentModule()
-
-        # TODO: Why do this accessing a node, do this outside.
-        return context.getCodeObjectHandle(
-            code_object  = self.code_object,
-            filename     = parent_module.getRunTimeFilename(),
-            line_number  = line_number,
-            is_optimized = is_optimized,
-            new_locals   = new_locals,
-            has_closure  = closure_provider.isExpressionFunctionBody() and \
-                           closure_provider.getClosureVariables() != () and \
-                           not closure_provider.isExpressionClassBody(),
-            future_flags = parent_module.getFutureSpec().asFlags()
-        )
-
     def computeStatementsSequence(self, trace_collection):
         # The extraction of parts of the frame that can be moved before or after
         # the frame scope, takes it toll to complexity, pylint: disable=too-many-branches
@@ -225,6 +212,12 @@ class StatementsFrame(StatementsSequence):
                     break
 
         if not new_statements:
+            trace_collection.signalChange(
+                "new_statements",
+                self.getSourceReference(),
+                "Removed empty frame object of '%s'." % self.code_object.getCodeObjectName()
+            )
+
             return None
 
         # If our statements changed just now, they are not immediately usable,
@@ -275,3 +268,89 @@ class StatementsFrame(StatementsSequence):
                 self.setStatements(new_statements)
 
             return self
+
+
+
+class StatementsFrameModule(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_MODULE"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "once",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return False
+
+
+class StatementsFrameFunction(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_FUNCTION"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "full",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return False
+
+
+class StatementsFrameGenerator(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_GENERATOR"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
+
+
+class StatementsFrameCoroutine(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_COROUTINE"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
+
+
+class StatementsFrameAsyncgen(StatementsFrameBase):
+    kind = "STATEMENTS_FRAME_ASYNCGEN"
+
+    def __init__(self, statements, code_object, source_ref):
+        StatementsFrameBase.__init__(
+            self,
+            statements  = statements,
+            code_object = code_object,
+            guard_mode  = "generator",
+            source_ref  = source_ref
+        )
+
+    @staticmethod
+    def hasStructureMember():
+        return True
