@@ -30,19 +30,19 @@ classes.
 """
 
 from nuitka import Options, Variables
-from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.FutureSpecs import fromFlags
-from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
 from nuitka.PythonVersions import python_version
 from nuitka.tree.Extractions import updateVariableUsage
 
 from .Checkers import checkStatementsSequenceOrNone
+from .CodeObjectSpecs import CodeObjectSpec
 from .ExpressionBases import (
     CompileTimeConstantExpressionBase,
     ExpressionBase,
     ExpressionChildrenHavingBase
 )
+from .FutureSpecs import fromFlags
 from .IndicatorMixins import (
+    EntryPointMixin,
     MarkLocalsDictIndicatorMixin,
     MarkUnoptimizedFunctionIndicatorMixin
 )
@@ -63,11 +63,14 @@ from .ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
 class MaybeLocalVariableUsage(Exception):
     pass
 
+
 class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
                                  ExpressionChildrenHavingBase):
 
-    def __init__(self, provider, name, code_prefix, is_class, flags, source_ref,
-                 body = None):
+    def __init__(self, provider, name, code_prefix, flags, source_ref, body):
+        while provider.isExpressionOutlineBody():
+            provider = provider.getParentVariableProvider()
+
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
@@ -78,8 +81,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
 
         ClosureTakerMixin.__init__(
             self,
-            provider      = provider,
-            early_closure = is_class
+            provider = provider
         )
 
         ClosureGiverNodeMixin.__init__(
@@ -90,7 +92,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
 
         # Special things, "has_super" indicates presence of "super" in variable
         # usage, which modifies some behaviors.
-        self.flags = flags
+        self.flags = flags or None
 
         # Hack: This allows some APIs to work although this is not yet
         # officially a child yet. Important during building.
@@ -105,14 +107,27 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         self.non_local_declarations = []
 
         # Register ourselves immediately with the module.
-        provider.getParentModule().addFunction(self)
 
-        self.trace_collection = None
+        # TODO: Have a base class that is between outline function and entry
+        # point functions.
+        if not self.isExpressionOutlineFunction() and \
+           not self.isExpressionClassBody():
+            provider.getParentModule().addFunction(self)
 
     @staticmethod
     def isExpressionFunctionBodyBase():
         return True
 
+    def getEntryPoint(self):
+        """ Entry point for code.
+
+            Normally ourselves. Only outlines will refer to their parent which
+            technically owns them.
+
+        """
+
+        return self
+
     def getContainingClassDictCreation(self):
         current = self
 
@@ -125,15 +140,29 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         return None
 
     def hasFlag(self, flag):
-        return flag in self.flags
+        return self.flags is not None and flag in self.flags
 
-    def getLocalsMode(self):
-        if python_version >= 300:
-            return "updated"
-        elif self.isEarlyClosure() or self.isUnoptimized():
-            return "updated"
-        else:
-            return "copy"
+    def discardFlag(self, flag):
+        if self.flags is not None:
+            self.flags.discard(flag)
+
+    @staticmethod
+    def isEarlyClosure():
+        """ Early closure taking means immediate binding of references.
+
+        Normally it's good to lookup name references immediately, but not for
+        functions. In case of a function body it is not allowed to do that,
+        because a later assignment needs to be queried first. Nodes need to
+        indicate via this if they would like to resolve references at the same
+        time as assignments.
+        """
+
+        return False
+
+    def isLocalsUpdatedMode(self):
+        return python_version >= 300 or \
+               self.isEarlyClosure() or \
+               self.isUnoptimized()
 
     def hasVariableName(self, variable_name):
         return variable_name in self.providing or variable_name in self.temp_variables
@@ -143,21 +172,42 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
 
     def getLocalVariables(self):
         return [
-            variable for
-            variable in
+            variable
+            for variable in
+            self.providing.values()
+            if variable.isLocalVariable()
+        ]
+
+    def getLocalVariableNames(self):
+        return [
+            variable.getName()
+            for variable in
             self.providing.values()
             if variable.isLocalVariable()
         ]
 
     def getUserLocalVariables(self):
         return tuple(
-            variable for
-            variable in
+            variable
+            for variable in
             self.providing.values()
             if variable.isLocalVariable() and not variable.isParameterVariable()
             if variable.getOwner() is self
         )
 
+    def getOutlineLocalVariables(self):
+        outlines = self.getTraceCollection().getOutlineFunctions()
+
+        if outlines is None:
+            return ()
+
+        result = []
+
+        for outline in outlines:
+            result.extend(outline.getUserLocalVariables())
+
+        return tuple(result)
+
     def removeClosureVariable(self, variable):
         assert variable in self.providing.values(), (self.providing, variable)
 
@@ -192,6 +242,9 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             new_variable = new_variable
         )
 
+    def hasClosureVariable(self, variable):
+        return variable in self.taken
+
     def removeUserVariable(self, variable):
         assert variable in self.providing.values(), (self.providing, variable)
 
@@ -211,7 +264,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
         return result
 
     def getVariableForReference(self, variable_name):
-        # print( "REF func", self.getCodeName(), variable_name )
+        # print( "REF func", self, variable_name )
 
         if self.hasProvidedVariable(variable_name):
             result = self.getProvidedVariable(variable_name)
@@ -225,12 +278,16 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             if not result.isModuleVariable():
                 self.registerProvidedVariable(result)
 
+            entry_point = self.getEntryPoint()
+
             # For "exec" containing/star import containing, we raise this exception to indicate
             # that instead of merely a variable, to be assigned, we need to replace with locals
             # dict access.
-            if python_version < 300 and not self.isExpressionClassBody() and \
+            if python_version < 300 and \
+               not entry_point.isExpressionClassBody() and \
+               not entry_point.isPythonMainModule() and \
                result.isModuleVariable() and \
-               self.isUnoptimized():
+               entry_point.isUnoptimized():
                 raise MaybeLocalVariableUsage
 
         return result
@@ -305,9 +362,27 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ClosureGiverNodeMixin,
             return self.getBody().mayRaiseException(exception_type)
 
 
+class ExpressionFunctionEntryPointBase(EntryPointMixin, ExpressionFunctionBodyBase):
+    def __init__(self, provider, name, code_prefix, flags, source_ref):
+        ExpressionFunctionBodyBase.__init__(
+            self,
+            provider    = provider,
+            name        = name,
+            code_prefix = code_prefix,
+            flags       = flags,
+            body        = None,
+            source_ref  = source_ref
+        )
+
+        EntryPointMixin.__init__(self)
+
+    getBody = ChildrenHavingMixin.childGetter("body")
+    setBody = ChildrenHavingMixin.childSetter("body")
+
+
 class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
                              MarkUnoptimizedFunctionIndicatorMixin,
-                             ExpressionFunctionBodyBase):
+                             ExpressionFunctionEntryPointBase):
     # We really want these many ancestors, as per design, we add properties via
     # base class mix-ins a lot, leading to many methods, pylint: disable=R0901
 
@@ -325,22 +400,16 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     if python_version >= 340:
         qualname_setup = None
 
-    def __init__(self, provider, name, doc, parameters, flags, source_ref,
-                 body = None):
-        while provider.isExpressionOutlineBody():
-            provider = provider.getParentVariableProvider()
-
+    def __init__(self, provider, name, doc, parameters, flags, source_ref):
         if name == "<listcontraction>":
             assert python_version >= 300
 
-        ExpressionFunctionBodyBase.__init__(
+        ExpressionFunctionEntryPointBase.__init__(
             self,
             provider    = provider,
             name        = name,
             code_prefix = "function",
-            is_class    = False,
             flags       = flags,
-            body        = body,
             source_ref  = source_ref
         )
 
@@ -432,9 +501,6 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     def getParameters(self):
         return self.parameters
 
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
-
     def needsCreation(self):
         return self.needs_creation
 
@@ -479,6 +545,7 @@ class ExpressionFunctionBody(MarkLocalsDictIndicatorMixin,
     def needsExceptionReturnValue(self):
         return self.return_exception
 
+
 def convertNoneConstantOrEmptyDictToNone(node):
     if node is None:
         return None
@@ -556,17 +623,19 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
-        code_object_specs = {}
+        code_object_args = {}
         other_args = {}
 
         for key, value in args.items():
             if key.startswith("co_"):
-                code_object_specs[key] = value
+                code_object_args[key] = value
+            elif key == "code_flags":
+                code_object_args["future_spec"] = fromFlags(args["code_flags"])
             else:
                 other_args[key] = value
 
-        if code_object_specs:
-            code_object = CodeObjectSpec(**code_object_specs)
+        if code_object_args:
+            code_object = CodeObjectSpec(**code_object_args)
         else:
             code_object = None
 
@@ -577,45 +646,6 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
         )
 
     def computeExpression(self, trace_collection):
-        defaults = self.getDefaults()
-
-        side_effects = []
-
-        for default in defaults:
-            if default.willRaiseException(BaseException):
-                result = wrapExpressionWithSideEffects(
-                    side_effects = side_effects,
-                    old_node     = self,
-                    new_node     = default
-                )
-
-                return result, "new_raise", "Default value contains raise."
-
-        kw_defaults = self.getKwDefaults()
-
-        if kw_defaults is not None:
-            if kw_defaults.willRaiseException(BaseException):
-                result = wrapExpressionWithSideEffects(
-                    side_effects = side_effects,
-                    old_node     = self,
-                    new_node     = kw_defaults
-                )
-
-                return result, "new_raise", "Keyword default values contain raise."
-
-            side_effects.append(kw_defaults)
-
-        annotations = self.getAnnotations()
-
-        if annotations is not None and annotations.willRaiseException(BaseException):
-            result = wrapExpressionWithSideEffects(
-                side_effects = side_effects,
-                old_node     = self,
-                new_node     = annotations
-            )
-
-            return result, "new_raise", "Annotation values contain raise."
-
         self.variable_closure_traces = []
 
         for closure_variable in self.getFunctionRef().getFunctionBody().getClosureVariables():
@@ -666,15 +696,11 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
 
         if call_args is None:
             args_tuple = ()
-        elif call_args.isExpressionConstantRef() or \
-             call_args.isExpressionMakeTuple():
-            args_tuple = call_args.getIterationValues()
         else:
-            # TODO: Can this even happen, i.e. does the above check make
-            # sense.
-            assert False, call_args
+            assert call_args.isExpressionConstantRef() or \
+                   call_args.isExpressionMakeTuple()
 
-            return call_node, None, None
+            args_tuple = call_args.getIterationValues()
 
         function_body = self.getFunctionRef().getFunctionBody()
 
@@ -736,10 +762,16 @@ error""" % self.getName()
         # TODO: Ought to use values. If they are all constant, how about we
         # assume no cost, pylint: disable=unused-argument
 
-        if not Options.isExperimental():
+        function_body = self.getFunctionRef().getFunctionBody()
+
+        if function_body.isExpressionClassBody():
+            if function_body.getBody().getStatements()[0].isStatementReturn():
+                return 0
+
             return None
 
-        function_body = self.getFunctionRef().getFunctionBody()
+        if True or not Options.isExperimental("function_inlining"):
+            return None
 
         if function_body.isExpressionGeneratorObjectBody():
             # TODO: That's not even allowed, is it?
@@ -747,9 +779,6 @@ error""" % self.getName()
 
             return None
 
-        if function_body.isExpressionClassBody():
-            return None
-
         # TODO: Lying for the demo, this is too limiting, but needs frames to
         # be allowed twice in a context.
         if function_body.mayRaiseException(BaseException):
@@ -758,6 +787,8 @@ error""" % self.getName()
         return 20
 
     def createOutlineFromCall(self, provider, values):
+        from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
+
         return convertFunctionCallToOutline(
             provider     = provider,
             function_ref = self.getFunctionRef(),
@@ -771,7 +802,6 @@ error""" % self.getName()
         ]
 
 
-
 class ExpressionFunctionRef(ExpressionBase):
     kind = "EXPRESSION_FUNCTION_REF"
 
@@ -833,12 +863,11 @@ class ExpressionFunctionRef(ExpressionBase):
         # in the reference. We should do it in the body, and there we should
         # limit us to only doing it once per module run, e.g. being based on
         # presence in used functions of the module already.
-        old_collection = function_body.trace_collection
-
-        function_body.trace_collection = TraceCollectionFunction(
+        trace_collection = TraceCollectionFunction(
             parent        = trace_collection,
             function_body = function_body
         )
+        old_collection = function_body.setTraceCollection(trace_collection)
 
         statements_sequence = function_body.getBody()
 
@@ -849,13 +878,13 @@ class ExpressionFunctionRef(ExpressionBase):
 
         if statements_sequence is not None:
             result = statements_sequence.computeStatementsSequence(
-                trace_collection = function_body.trace_collection
+                trace_collection = trace_collection
             )
 
             if result is not statements_sequence:
                 function_body.setBody(result)
 
-        function_body.trace_collection.updateFromCollection(old_collection)
+        trace_collection.updateVariablesFromCollection(old_collection)
 
         # TODO: Function collection may now know something.
         return self, None, None
@@ -913,7 +942,11 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
                 values   = values
             )
 
-            return result, "new_statements", "Function call in-lined."
+            return (
+                result,
+                "new_statements",
+                "Function call to '%s' in-lined." % function_body.getCodeName()
+            )
 
         self.variable_closure_traces = []
 
@@ -952,7 +985,7 @@ class ExpressionFunctionCall(ExpressionChildrenHavingBase):
 class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionBase):
     kind = "EXPRESSION_FUNCTION_QUALNAME_REF"
 
-    __slots__ = "function_body",
+    __slots__ = ("function_body",)
 
     def __init__(self, function_body, source_ref):
         CompileTimeConstantExpressionBase.__init__(
