@@ -46,6 +46,10 @@ class ShapeTypeNoneType(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeBool(ShapeBase):
     @staticmethod
@@ -73,6 +77,10 @@ class ShapeTypeBool(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeInt(ShapeBase):
     @staticmethod
@@ -95,6 +103,10 @@ class ShapeTypeInt(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeLong(ShapeBase):
     @staticmethod
@@ -117,6 +129,11 @@ class ShapeTypeLong(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
+
 if python_version < 300:
     class ShapeTypeIntOrLong(ShapeBase):
         @staticmethod
@@ -134,6 +151,10 @@ if python_version < 300:
         @staticmethod
         def hasShapeSlotNext():
             return False
+
+        @staticmethod
+        def hasShapeSlotContains():
+            return False
 else:
     ShapeTypeIntOrLong = ShapeTypeInt
 
@@ -159,6 +180,10 @@ class ShapeTypeFloat(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeTuple(ShapeBase):
     @staticmethod
@@ -185,6 +210,10 @@ class ShapeTypeTuple(ShapeBase):
     def getShapeIter():
         return ShapeTypeTupleIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeTupleIterator(ShapeIterator):
     @staticmethod
@@ -221,6 +250,10 @@ class ShapeTypeList(ShapeBase):
     def getShapeIter():
         return ShapeTypeListIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeListIterator(ShapeIterator):
     @staticmethod
@@ -257,6 +290,10 @@ class ShapeTypeSet(ShapeBase):
     def getShapeIter():
         return ShapeTypeSetIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeSetIterator(ShapeIterator):
     @staticmethod
@@ -268,6 +305,36 @@ class ShapeTypeSetIterator(ShapeIterator):
         return False
 
 
+class ShapeTypeFrozenset(ShapeBase):
+    @staticmethod
+    def getTypeName():
+        return "frozenset"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return True
+
+    @staticmethod
+    def hasShapeSlotInt():
+        return False
+
+    @staticmethod
+    def hasShapeSlotNext():
+        return False
+
+    @staticmethod
+    def hasShapeSlotIter():
+        return True
+
+    @staticmethod
+    def getShapeIter():
+        return ShapeTypeSetIterator
+
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
+
 class ShapeTypeDict(ShapeBase):
     @staticmethod
     def getTypeName():
@@ -293,6 +360,10 @@ class ShapeTypeDict(ShapeBase):
     def getShapeIter():
         return ShapeTypeDictIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeDictIterator(ShapeIterator):
     @staticmethod
@@ -329,6 +400,10 @@ class ShapeTypeStr(ShapeBase):
     def getShapeIter():
         return ShapeTypeStrIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeStrIterator(ShapeIterator):
     @staticmethod
@@ -366,6 +441,10 @@ if python_version < 300:
         def getShapeIter():
             return ShapeTypeUnicodeIterator
 
+        @staticmethod
+        def hasShapeSlotContains():
+            return True
+
     class ShapeTypeUnicodeIterator(ShapeIterator):
         @staticmethod
         def getTypeName():
@@ -395,6 +474,10 @@ if python_version < 300:
         @staticmethod
         def hasShapeSlotNext():
             return False
+
+        @staticmethod
+        def hasShapeSlotContains():
+            return True
 else:
     ShapeTypeStrOrUnicode = ShapeTypeStr
 
@@ -425,6 +508,10 @@ if python_version >= 300:
         def getShapeIter():
             return ShapeTypeBytesIterator
 
+        @staticmethod
+        def hasShapeSlotContains():
+            return True
+
 
     class ShapeTypeBytesIterator(ShapeIterator):
         @staticmethod
@@ -440,6 +527,46 @@ else:
     ShapeTypeBytesIterator = ShapeTypeStrIterator
 
 
+class ShapeTypeBytearray(ShapeBase):
+    @staticmethod
+    def getTypeName():
+        return "bytearray"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return True
+
+    @staticmethod
+    def hasShapeSlotInt():
+        return False
+
+    @staticmethod
+    def hasShapeSlotIter():
+        return True
+
+    @staticmethod
+    def hasShapeSlotNext():
+        return False
+
+    @staticmethod
+    def getShapeIter():
+        return ShapeTypeBytearrayIterator
+
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
+
+class ShapeTypeBytearrayIterator(ShapeIterator):
+    @staticmethod
+    def getTypeName():
+        return "bytearray_iterator"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return False
+
+
 class ShapeTypeEllipsisType(ShapeBase):
     @staticmethod
     def getTypeName():
@@ -461,6 +588,10 @@ class ShapeTypeEllipsisType(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeSlice(ShapeBase):
     @staticmethod
@@ -483,6 +614,10 @@ class ShapeTypeSlice(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeXrange(ShapeBase):
     @staticmethod
@@ -509,6 +644,10 @@ class ShapeTypeXrange(ShapeBase):
     def getShapeIter():
         return ShapeTypeXrangeIterator
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
 
 class ShapeTypeXrangeIterator(ShapeIterator):
     @staticmethod
@@ -541,6 +680,10 @@ class ShapeTypeType(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeModule(ShapeBase):
     @staticmethod
@@ -567,6 +710,88 @@ class ShapeTypeModule(ShapeBase):
     def hasShapeSlotNext():
         return False
 
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
 
 class ShapeTypeBuiltinModule(ShapeTypeModule):
     pass
+
+
+class ShapeTypeFile(ShapeBase):
+    @staticmethod
+    def getTypeName():
+        return "file"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return False
+
+    @staticmethod
+    def hasShapeSlotInt():
+        return False
+
+    @staticmethod
+    def hasShapeSlotIter():
+        return True
+
+    @staticmethod
+    def hasShapeSlotNext():
+        return True
+
+    @staticmethod
+    def hasShapeSlotContains():
+        return True
+
+
+class ShapeTypeStaticmethod(ShapeBase):
+    @staticmethod
+    def getTypeName():
+        return "staticmethod"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return False
+
+    @staticmethod
+    def hasShapeSlotInt():
+        return False
+
+    @staticmethod
+    def hasShapeSlotIter():
+        return False
+
+    @staticmethod
+    def hasShapeSlotNext():
+        return False
+
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
+
+
+class ShapeTypeClassmethod(ShapeBase):
+    @staticmethod
+    def getTypeName():
+        return "classmethod"
+
+    @staticmethod
+    def hasShapeSlotLen():
+        return False
+
+    @staticmethod
+    def hasShapeSlotInt():
+        return False
+
+    @staticmethod
+    def hasShapeSlotIter():
+        return False
+
+    @staticmethod
+    def hasShapeSlotNext():
+        return False
+
+    @staticmethod
+    def hasShapeSlotContains():
+        return False
