@@ -35,7 +35,7 @@ from nuitka.nodes.BuiltinRefNodes import (
     ExpressionBuiltinAnonymousRef,
     makeExpressionBuiltinRef
 )
-from nuitka.nodes.CallNodes import ExpressionCall, ExpressionCallNoKeywords
+from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.ClassNodes import (
     ExpressionClassBody,
     ExpressionSelectMetaclass
@@ -52,13 +52,9 @@ from nuitka.nodes.DictionaryNodes import (
     ExpressionDictOperationGet,
     StatementDictOperationRemove
 )
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
-    ExpressionFunctionQualnameRef,
-    ExpressionFunctionRef
-)
+from nuitka.nodes.FunctionNodes import ExpressionFunctionQualnameRef
 from nuitka.nodes.GlobalsLocalsNodes import (
+    ExpressionBuiltinLocalsCopy,
     ExpressionBuiltinLocalsUpdated,
     StatementSetLocals
 )
@@ -74,18 +70,19 @@ from nuitka.nodes.VariableRefNodes import (
 )
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
     buildFrameNode,
     buildNode,
     buildNodeList,
     extractDocFromBody,
     getKind,
-    makeDictCreationOrConstant,
+    makeDictCreationOrConstant2,
     makeSequenceCreationOrConstant,
     makeStatementsSequence,
-    makeStatementsSequenceFromStatement
+    makeStatementsSequenceFromStatement,
+    mangleName
 )
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def _buildClassNode3(provider, node, source_ref):
@@ -98,8 +95,7 @@ def _buildClassNode3(provider, node, source_ref):
 
     # We need a scope for the temporary variables, and they might be closured.
     temp_scope = provider.allocateTempScope(
-        name          = "class_creation",
-        allow_closure = True
+        name = "class_creation"
     )
 
     tmp_bases = provider.allocateTempVariable(
@@ -123,7 +119,6 @@ def _buildClassNode3(provider, node, source_ref):
         provider   = provider,
         name       = node.name,
         doc        = class_doc,
-        flags      = set(),
         source_ref = source_ref
     )
 
@@ -245,7 +240,7 @@ def _buildClassNode3(provider, node, source_ref):
     statements += [
         StatementAssignmentVariableName(
             variable_name = "__class__",
-            source        = ExpressionCall(
+            source        = makeExpressionCall(
                 called     = ExpressionTempVariableRef(
                     variable   = tmp_metaclass,
                     source_ref = source_ref
@@ -302,28 +297,14 @@ def _buildClassNode3(provider, node, source_ref):
     # returns its created class and cannot have other return statements
     # contained.
 
-    decorated_body = ExpressionFunctionCall(
-        function   = ExpressionFunctionCreation(
-            function_ref = ExpressionFunctionRef(
-                function_body = class_creation_function,
-                source_ref    = source_ref
-            ),
-            code_object  = code_object,
-            defaults     = (),
-            kw_defaults  = None,
-            annotations  = None,
-            source_ref   = source_ref
-        ),
-        values     = (),
-        source_ref = source_ref
-    )
+    decorated_body = class_creation_function
 
     for decorator in buildNodeList(
             provider,
             reversed(node.decorator_list),
             source_ref
         ):
-        decorated_body = ExpressionCallNoKeywords(
+        decorated_body = makeExpressionCall(
             called     = decorator,
             args       = ExpressionMakeTuple(
                 elements   = (
@@ -331,6 +312,7 @@ def _buildClassNode3(provider, node, source_ref):
                 ),
                 source_ref = source_ref
             ),
+            kw         = None,
             source_ref = decorator.getSourceReference()
         )
 
@@ -350,13 +332,9 @@ def _buildClassNode3(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable   = tmp_class_decl_dict,
-            source     = makeDictCreationOrConstant(
+            source     = makeDictCreationOrConstant2(
                 keys       = [
-                    makeConstantRefNode(
-                        constant      = keyword.arg,
-                        source_ref    = source_ref,
-                        user_provided = True
-                    )
+                    keyword.arg
                     for keyword in
                     node.keywords
                 ],
@@ -483,7 +461,7 @@ def _buildClassNode3(provider, node, source_ref):
                     source_ref    = source_ref,
                     user_provided = True
                 ),
-                expression_yes = ExpressionCall(
+                expression_yes = makeExpressionCall(
                     called     = ExpressionAttributeLookup(
                         source         = ExpressionTempVariableRef(
                             variable   = tmp_metaclass,
@@ -517,7 +495,9 @@ def _buildClassNode3(provider, node, source_ref):
             source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable   = provider.getVariableForAssignment(node.name),
+            variable   = provider.getVariableForAssignment(
+                mangleName(node.name, provider)
+            ),
             source     = decorated_body,
             source_ref = source_ref
         ),
@@ -562,7 +542,6 @@ def _buildClassNode2(provider, node, source_ref):
         provider   = provider,
         name       = node.name,
         doc        = class_doc,
-        flags      = set(),
         source_ref = source_ref
     )
 
@@ -587,6 +566,7 @@ def _buildClassNode2(provider, node, source_ref):
         code_object = code_object,
         source_ref  = source_ref
     )
+
     if body is not None:
         # The frame guard has nothing to tell its line number to.
         body.source_ref = source_ref.atInternal()
@@ -622,7 +602,7 @@ def _buildClassNode2(provider, node, source_ref):
     statements += [
         body,
         StatementReturn(
-            expression = ExpressionBuiltinLocalsUpdated(
+            expression = ExpressionBuiltinLocalsCopy(
                 source_ref = source_ref
             ),
             source_ref = source_ref.atInternal()
@@ -771,21 +751,7 @@ def _buildClassNode2(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable   = tmp_class_dict,
-            source     =   ExpressionFunctionCall(
-                function   = ExpressionFunctionCreation(
-                    function_ref = ExpressionFunctionRef(
-                        function_body = function_body,
-                        source_ref    = source_ref
-                    ),
-                    code_object  = None,
-                    defaults     = (),
-                    kw_defaults  = None,
-                    annotations  = None,
-                    source_ref   = source_ref
-                ),
-                values     = (),
-                source_ref = source_ref
-            ),
+            source     = function_body,
             source_ref = source_ref
         ),
         StatementAssignmentVariable(
@@ -822,7 +788,7 @@ def _buildClassNode2(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable   = tmp_class,
-            source     = ExpressionCallNoKeywords(
+            source     = makeExpressionCall(
                 called     = ExpressionTempVariableRef(
                     variable   = tmp_metaclass,
                     source_ref = source_ref
@@ -845,6 +811,7 @@ def _buildClassNode2(provider, node, source_ref):
                     ),
                     source_ref = source_ref
                 ),
+                kw         = None,
                 source_ref = source_ref
             ),
             source_ref = source_ref
@@ -859,7 +826,7 @@ def _buildClassNode2(provider, node, source_ref):
         statements.append(
             StatementAssignmentVariable(
                 variable   = tmp_class,
-                source     = ExpressionCallNoKeywords(
+                source     = makeExpressionCall(
                     called     = decorator,
                     args       = ExpressionMakeTuple(
                         elements   = (
@@ -870,6 +837,7 @@ def _buildClassNode2(provider, node, source_ref):
                         ),
                         source_ref = source_ref
                     ),
+                    kw         = None,
                     source_ref = decorator.getSourceReference()
                 ),
                 source_ref = decorator.getSourceReference()
@@ -878,7 +846,7 @@ def _buildClassNode2(provider, node, source_ref):
 
     statements.append(
         StatementAssignmentVariableName(
-            variable_name = node.name,
+            variable_name = mangleName(node.name, provider),
             source        = ExpressionTempVariableRef(
                 variable   = tmp_class,
                 source_ref = source_ref
