@@ -23,9 +23,8 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTargetVariableRef,
     StatementAssignmentVariable,
+    StatementAssignmentVariableName,
     StatementReleaseVariable
 )
 from nuitka.nodes.AsyncgenNodes import (
@@ -37,14 +36,8 @@ from nuitka.nodes.BuiltinIteratorNodes import (
     StatementSpecialUnpackCheck
 )
 from nuitka.nodes.BuiltinNextNodes import ExpressionSpecialUnpack
-from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinRef
-from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
+from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinRef
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.ConstantRefNodes import (
-    ExpressionConstantNoneRef,
-    makeConstantRefNode
-)
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.CoroutineNodes import (
     ExpressionCoroutineObjectBody,
     ExpressionMakeCoroutineObject
@@ -58,40 +51,33 @@ from nuitka.nodes.FunctionNodes import (
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
     ExpressionMakeGeneratorObject,
-    StatementGeneratorReturn
+    StatementGeneratorReturnNone
 )
 from nuitka.nodes.ParameterSpecs import ParameterSpec
-from nuitka.nodes.ReturnNodes import StatementReturn
+from nuitka.nodes.ReturnNodes import StatementReturn, StatementReturnNone
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableRef
+    ExpressionVariableNameRef
 )
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import (
     buildFrameNode,
     buildNode,
     buildNodeList,
     detectFunctionBodyKind,
     extractDocFromBody,
     getKind,
-    makeDictCreationOrConstant,
+    makeCallNode,
+    makeDictCreationOrConstant2,
     makeStatementsSequenceFromStatement,
     mangleName
 )
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
-from .SyntaxErrors import raiseSyntaxError
 
 
-def _insertFinalReturnStatement(function_statements_body, return_class,
-                                source_ref):
-    return_statement = return_class(
-        expression = ExpressionConstantNoneRef(
-            source_ref = source_ref
-        ),
-        source_ref = source_ref
-    )
-
+def _insertFinalReturnStatement(function_statements_body, return_statement):
     if function_statements_body is None:
         function_statements_body = makeStatementsSequenceFromStatement(
             statement = return_statement
@@ -137,6 +123,7 @@ def buildFunctionNode(provider, node, source_ref):
             flags      = flags,
             source_ref = source_ref
         )
+        code_body.qualname_provider = provider
 
         for variable in function_body.getVariables():
             code_body.getVariableForReference(variable.getName())
@@ -190,8 +177,9 @@ def buildFunctionNode(provider, node, source_ref):
         # TODO: Generators might have to raise GeneratorExit instead.
         function_statements_body = _insertFinalReturnStatement(
             function_statements_body = function_statements_body,
-            return_class             = StatementReturn,
-            source_ref               = source_ref
+            return_statement         = StatementReturnNone(
+                source_ref = source_ref
+            )
         )
 
     if function_statements_body.isStatementsFrame():
@@ -225,12 +213,12 @@ def buildFunctionNode(provider, node, source_ref):
        provider.isExpressionClassBody():
 
         for decorator in decorators:
-            if decorator.isExpressionVariableRef() and \
+            if decorator.isExpressionVariableNameRef() and \
                decorator.getVariableName() == "staticmethod":
                 break
         else:
             decorators.append(
-                ExpressionBuiltinRef(
+                makeExpressionBuiltinRef(
                     builtin_name = "staticmethod",
                     source_ref   = source_ref
                 )
@@ -241,12 +229,12 @@ def buildFunctionNode(provider, node, source_ref):
        provider.isExpressionClassBody():
 
         for decorator in decorators:
-            if decorator.isExpressionVariableRef() and \
+            if decorator.isExpressionVariableNameRef() and \
                decorator.getVariableName() == "classmethod":
                 break
         else:
             decorators.append(
-                ExpressionBuiltinRef(
+                makeExpressionBuiltinRef(
                     builtin_name = "classmethod",
                     source_ref   = source_ref
                 )
@@ -254,26 +242,20 @@ def buildFunctionNode(provider, node, source_ref):
 
     decorated_function = function_creation
     for decorator in decorators:
-        decorated_function = ExpressionCallNoKeywords(
-            called     = decorator,
-            args       = ExpressionMakeTuple(
-                elements   = (decorated_function,),
-                source_ref = source_ref
-            ),
-            source_ref = decorator.getSourceReference()
+        decorated_function = makeCallNode(
+            decorator,
+            decorated_function,
+            decorator.getSourceReference()
         )
 
-    result = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = mangleName(node.name, provider),
-            source_ref    = source_ref
-        ),
-        source       = decorated_function,
-        source_ref   = source_ref
+    result = StatementAssignmentVariableName(
+        variable_name = mangleName(node.name, provider),
+        source        = decorated_function,
+        source_ref    = source_ref
     )
 
     if python_version >= 340:
-        function_body.qualname_setup = result.getTargetVariableRef()
+        function_body.qualname_setup = result.getVariableName()
 
     return result
 
@@ -315,6 +297,8 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             source_ref = source_ref
         )
 
+    for variable in creator_function_body.getVariables():
+        function_body.getVariableForReference(variable.getName())
 
     decorators = buildNodeList(
         provider   = provider,
@@ -337,8 +321,9 @@ def buildAsyncFunctionNode(provider, node, source_ref):
 
     function_statements_body = _insertFinalReturnStatement(
         function_statements_body = function_statements_body,
-        return_class             = StatementGeneratorReturn,
-        source_ref               = source_ref
+        return_statement         = StatementGeneratorReturnNone(
+            source_ref = source_ref
+        )
     )
 
     if function_statements_body.isStatementsFrame():
@@ -402,26 +387,19 @@ def buildAsyncFunctionNode(provider, node, source_ref):
 
     decorated_function = function_creation
     for decorator in decorators:
-        decorated_function = ExpressionCallNoKeywords(
-            called     = decorator,
-            args       = ExpressionMakeTuple(
-                elements   = (decorated_function,),
-                source_ref = source_ref
-            ),
-            source_ref = decorator.getSourceReference()
+        decorated_function = makeCallNode(
+            decorator,
+            decorated_function,
+            decorator.getSourceReference()
         )
 
-
-    result = StatementAssignmentVariable(
-        variable_ref = ExpressionTargetVariableRef(
-            variable_name = mangleName(node.name, provider),
-            source_ref    = source_ref
-        ),
-        source       = decorated_function,
-        source_ref   = source_ref
+    result = StatementAssignmentVariableName(
+        variable_name = mangleName(node.name, provider),
+        source        = decorated_function,
+        source_ref    = source_ref
     )
 
-    function_body.qualname_setup = result.getTargetVariableRef()
+    function_body.qualname_setup = result.getVariableName()
 
     # Share the non-local declarations. TODO: This may also apply to generators
     # and async generators.
@@ -444,17 +422,12 @@ def buildParameterKwDefaults(provider, node, function_body, source_ref):
             for kw_only_name, kw_default in \
               zip(kw_only_names, node.args.kw_defaults):
                 if kw_default is not None:
-                    keys.append(
-                        makeConstantRefNode(
-                            constant   = kw_only_name,
-                            source_ref = source_ref
-                        )
-                    )
+                    keys.append(kw_only_name)
                     values.append(
                         buildNode(provider, kw_default, source_ref)
                     )
 
-            kw_defaults = makeDictCreationOrConstant(
+            kw_defaults = makeDictCreationOrConstant2(
                 keys       = keys,
                 values     = values,
                 source_ref = source_ref
@@ -474,7 +447,6 @@ def buildParameterAnnotations(provider, node, source_ref):
     if python_version < 300:
         return None
 
-
     # Starting with Python 3.4, the names of parameters are mangled in
     # annotations as well.
     if python_version < 340:
@@ -486,13 +458,7 @@ def buildParameterAnnotations(provider, node, source_ref):
     values = []
 
     def addAnnotation(key, value):
-        keys.append(
-            makeConstantRefNode(
-                constant      = mangle(key),
-                source_ref    = source_ref,
-                user_provided = True
-            )
-        )
+        keys.append(mangle(key))
         values.append(value)
 
     def extractArg(arg):
@@ -548,7 +514,7 @@ def buildParameterAnnotations(provider, node, source_ref):
         )
 
     if keys:
-        return makeDictCreationOrConstant(
+        return makeDictCreationOrConstant2(
             keys       = keys,
             values     = values,
             source_ref = source_ref
@@ -622,6 +588,8 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
             source_ref.atColumnNumber(node.col_offset),
         )
 
+    parent_module = provider.getParentModule()
+
     code_object = CodeObjectSpec(
         co_name           = name,
         co_kind           = function_kind,
@@ -629,7 +597,10 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
         co_argcount       = parameters.getArgumentCount(),
         co_kwonlyargcount = parameters.getKwOnlyParameterCount(),
         co_has_starlist   = parameters.getStarListArgumentName() is not None,
-        co_has_stardict   = parameters.getStarDictArgumentName() is not None
+        co_has_stardict   = parameters.getStarDictArgumentName() is not None,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
     )
 
     outer_body = ExpressionFunctionBody(
@@ -660,15 +631,12 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
             statements.append(
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = iter_var,
-                        source_ref = source_ref
-                    ),
-                    source       = ExpressionBuiltinIter1(
+                    variable   = iter_var,
+                    source     = ExpressionBuiltinIter1(
                         value      = source,
                         source_ref = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 )
             )
 
@@ -680,11 +648,8 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
                     statements.append(
                         StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = arg_var,
-                                source_ref = source_ref
-                            ),
-                            source       = ExpressionSpecialUnpack(
+                            variable   = arg_var,
+                            source     = ExpressionSpecialUnpack(
                                 value      = ExpressionTempVariableRef(
                                     variable   = iter_var,
                                     source_ref = source_ref
@@ -693,7 +658,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                                 expected   = len(arg_names),
                                 source_ref = source_ref
                             ),
-                            source_ref   = source_ref
+                            source_ref = source_ref
                         )
                     )
 
@@ -738,7 +703,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
         for arg_name in parameters.getParameterNames():
             if arg_name.startswith('.'):
-                source = ExpressionVariableRef(
+                source = ExpressionVariableNameRef(
                     variable_name = arg_name,
                     source_ref    = source_ref
                 )
@@ -748,7 +713,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                 )
             else:
                 values.append(
-                    ExpressionVariableRef(
+                    ExpressionVariableNameRef(
                         variable_name = arg_name,
                         source_ref    = source_ref
                     )
@@ -806,8 +771,11 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
                             source_ref = source_ref
                         )
                         for variable in
-                        outer_body.getTempVariables()
-                    ]   ,
+                        sorted(
+                            outer_body.getTempVariables(),
+                            key = lambda variable: variable.getName()
+                        )
+                    ],
                     source_ref = source_ref,
                     public_exc = False
                 )
@@ -820,11 +788,7 @@ def buildFunctionWithParsing(provider, function_kind, name, function_doc, flags,
 
 
 def addFunctionVariableReleases(function):
-    assert function.isExpressionFunctionBody() or \
-           function.isExpressionClassBody() or \
-           function.isExpressionGeneratorObjectBody() or \
-           function.isExpressionCoroutineObjectBody() or \
-           function.isExpressionAsyncgenObjectBody()
+    assert function.isExpressionFunctionBodyBase()
 
     releases = []
 
