@@ -29,17 +29,17 @@ from nuitka.nodes.AssignNodes import (
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import StatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantNoneRef
-from nuitka.nodes.ImportNodes import ExpressionImportModuleHard
+from nuitka.nodes.ImportNodes import ExpressionImportModuleNameHard
 from nuitka.nodes.PrintNodes import StatementPrintNewline, StatementPrintValue
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
-from .Helpers import (
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
     buildNode,
     buildNodeList,
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements
 )
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def buildPrintNode(provider, node, source_ref):
@@ -55,7 +55,7 @@ def buildPrintNode(provider, node, source_ref):
 
         target_default_statement = StatementAssignmentVariable(
             variable   = tmp_target_variable,
-            source     = ExpressionImportModuleHard(
+            source     = ExpressionImportModuleNameHard(
                 module_name = "sys",
                 import_name = "stdout",
                 source_ref  = source_ref
